"""
See main.py.
"""

# Copyright (c) 2013 contributors; see AUTHORS.
# Licensed under the Apache License, Version 2.0
# https://www.apache.org/licenses/LICENSE-2.0

from gossamer.integration import run_gossamerfile

__all__ = ['run_gossamerfile', ]

__version__ = '0.9.3'

